@@ -32,10 +32,10 @@ class ValidationError(object):
 
     def __init__(self, code, message):
         self.code = code
-        self.message = message
+        self.message = to_unicode(message) or u''
 
     def __repr__(self):
-        return '<ValidationError %s: %s>' % (self.code, self.message)
+        return self.message
 
 
 #- Real fields
@@ -44,74 +44,95 @@ class ValidationError(object):
 class _Field(object):
     """The real form field class.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
     Any other named parameter will be stored in `self.extra`.
     """
 
-    _value = u''
+    _value = None
+    python_value = None
+    original_value = None
     error = None
     name = 'unnamed'
 
-    hide_value = False
     has_changed = False
+    hide_value = False
 
-    def __init__(self, *validate, **kwargs):
+    def __init__(self, validate=None, **kwargs):
+        validate = validate or []
         self.validators = [val() if inspect.isclass(val) else val
             for val in validate]
         self.optional = not self._validator_in(v.Required, validate)
         # Extensibility FTW
         self.extra = kwargs
 
+    def reset(self):
+        self._value = None
+        self.python_value = None
+        self.original_value = None
+
     def _validator_in(self, validator, validators):
         for v in validators:
             if (v == validator) or isinstance(v, validator):
                 return True
         return False
 
-    def set_locale(self, locale='en', tz=utc):
+    def _init(self, data=None, original_value=None, files=None,
+            locale='en', tz=utc):
+        self.value = data or files
+        self.python_value = original_value
+        self.original_value = original_value
         self.locale = locale
         self.tz = tz
 
-    def load_value(self, python_value):
-        self._value = self.to_html(python_value)
-
     def _get_value(self):
-        return self._value if self._value and not self.hide_value else u''
+        if self.hide_value:
+            return u''
+        if not self._value:
+            return self.to_html()
+        return self._value
 
     def _set_value(self, value):
         if isinstance(value, list):
             value = value[0] if value else u''
         elif value is None:
             value = u''
-        self._value = value.strip()
+        if isinstance(value, basestring):
+            value = value.strip()
+        self._value = value
 
     value = property(_get_value, _set_value)
 
-    def to_html(self, python_value):
-        return to_unicode(python_value or u'')
+    def to_html(self):
+        return to_unicode(self.python_value or u'')
 
     def to_python(self):
-        if self._value is None:
+        if not self._value:
             return None
-        value = self._value.strip()
-        if value == u'':
-            return None 
-        return to_unicode(value)
+        return self._value
+
+    def clean_value(self, python_value):
+        return python_value
 
     def validate(self, cleaned_data=None):
         """Validates the current value of a field.
         """
-        self.error = None
         if cleaned_data is None:
-            python_value = self.to_python()
+            self.error = None
+            python_value = self.to_python() or self.python_value
+            python_value = self.clean_value(python_value)
+
+            if isinstance(python_value, ValidationError):
+                self.error = python_value
+                return
+
+            self.has_changed = python_value != self.original_value
             # Do not validate optional fields
             if (python_value is None) and self.optional:
                 return None
-            self._validate_value(python_value)
-            return python_value
+            return self._validate_value(python_value)
         self._validate_form(cleaned_data)
 
     def _validate_value(self, python_value):
@@ -120,13 +141,14 @@ class _Field(object):
                 continue
             if not val(python_value):
                 self.error = ValidationError(val.code, val.message)
-                break
+                return
+        return python_value
 
     def _validate_form(self, cleaned_data):
         for val in self.validators:
             if not isinstance(val, v.FormValidator):
                 continue
-            if not val(python_value):
+            if not val(cleaned_data):
                 self.error = ValidationError(val.code, val.message)
                 break
 
@@ -203,7 +225,7 @@ class _Text(_Field):
         An optional function that takes the value and return a 'cleaned'
         version of it. If the value raise an exception, `None` will be
         returned instead.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -212,13 +234,14 @@ class _Text(_Field):
     _type = 'text'
     _default_validator = None
 
-    def __init__(self, *validate, **kwargs):
+    def __init__(self, validate=None, **kwargs):
+        validate = validate or []
         defval = self._default_validator
         validate = list(validate)
         if defval and not self._validator_in(defval, validate):
             validate.append(defval())
 
-        super(_Text, self).__init__(*validate, **kwargs)
+        super(_Text, self).__init__(validate=validate, **kwargs)
 
     def __call__(self, **kwargs):
         return self.as_input(**kwargs)
@@ -245,7 +268,7 @@ class _Password(_Text):
     :param hide_value:
         If `True` this field will not reproduce the value on a form
         submit by default. This is the default for security purposes.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -253,15 +276,16 @@ class _Password(_Text):
     """
     _type = 'password'
 
-    def __init__(self, hide_value=True, *validate, **kwargs):
+    def __init__(self, hide_value=True, validate=None, **kwargs):
+        validate = validate or []
         self.hide_value = hide_value
-        super(_Password, self).__init__(*validate, **kwargs)
+        super(_Password, self).__init__(validate=validate, **kwargs)
 
 
 class _Number(_Text):
     """A number field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -280,7 +304,7 @@ class _Number(_Text):
 class _NaturalNumber(_Text):
     """A natural number (positive integer including zero) field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -299,7 +323,7 @@ class _NaturalNumber(_Text):
 class _Email(_Text):
     """An email field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -312,7 +336,7 @@ class _Email(_Text):
 class _URL(_Text):
     """An URL field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -325,23 +349,23 @@ class _URL(_Text):
 class _Date(_Text):
     """A date field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
     Any other named parameter will be stored in `self.extra`.
     """
-    _type = 'datetime'
+    _type = 'date'
     _default_validator = v.IsDate
 
-    def to_html(self, python_value, locale=None):
+    def to_html(self, locale=None):
         locale = locale or self.locale or 'en'
         try:
-            return format_date(python_value, locale=locale)
+            return format_date(self.python_value, locale=locale)
         except Exception:
             return u''
 
-    def to_python(locale=None):
+    def to_python(self, locale=None):
         if not self._value:
             return None
         locale = locale or self.locale or 'en'
@@ -354,7 +378,7 @@ class _Date(_Text):
 class _DateTime(_Text):
     """A datetime field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -363,16 +387,16 @@ class _DateTime(_Text):
     _type = 'datetime'
     _default_validator = v.IsDate
 
-    def to_html(self, python_value, locale=None, tz=None):
+    def to_html(self, locale=None, tz=None):
         locale = locale or self.locale
         tz = tz or self.tz
         try:
-            dt = python_value.astimezone(tz) if tz else python_value
+            dt = self.python_value.astimezone(tz) if tz else self.python_value
             return format_datetime(dt, locale=locale)
         except Exception:
             return u''
 
-    def to_python(locale=None):
+    def to_python(self, locale=None):
         if not self._value:
             return None
         locale = locale or self.locale
@@ -386,7 +410,7 @@ class _DateTime(_Text):
 class _Color(_Text):
     """A color field.
 
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -401,7 +425,7 @@ class _Color(_Text):
         r'(?:,(?P<a>[0-9]+))?\)',
         re.IGNORECASE)
 
-    def to_python():
+    def to_python(self):
         if not self._value:
             return None
         m = self._re_colors.match(self._value.replace(' ', ''))
@@ -448,7 +472,7 @@ class _File(_Field):
     :param upload:
         Optional function to be call for doing the actual file upload. It must
         return a python value ready for validation.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
@@ -456,16 +480,23 @@ class _File(_Field):
     """
     hide_value = True
 
-    def __init__(self, upload=None, *validate, **kwargs):
+    def __init__(self, upload=None, validate=None, **kwargs):
+        validate = validate or []
         self.upload = upload
-        super(_File, self).__init__(*validate, **kwargs)
+        super(_File, self).__init__(validate=validate, **kwargs)
 
-    def to_python(self, value):
-        if not self.value:
-            return None
-        if self.upload:
-            return self.upload(value)
-        return value
+    def to_html(self):
+        return self.python_value
+
+    def to_python(self):
+        if not self._value:
+            return self.original_value
+        if not self.upload:
+            return self._value
+        try:
+            return self.upload(self._value)
+        except Exception, e:
+            return ValidationError('invalid_file', str(e))
 
     def __call__(self, **kwargs):
         return self.as_input(**kwargs)
@@ -485,16 +516,17 @@ class _Boolean(_Field):
 
     :param falsy:
         A list of raw values considered `False`.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
     Any other named parameter will be stored in `self.extra`.
     """
 
-    def __init__(self, falsy=FALSY_VALUES, *validate, **kwargs):
+    def __init__(self, falsy=FALSY_VALUES, validate=None, **kwargs):
+        validate = validate or []
         self.falsy = falsy
-        super(_Boolean, self).__init__(*validate, **kwargs)
+        super(_Boolean, self).__init__(validate=validate, **kwargs)
 
     def to_python(self, value):
         if not value or (value in self.falsy):
@@ -525,17 +557,18 @@ class _Select(_Field):
         An optional function that takes the value and return a 'cleaned'
         version of it. If the value raise an exception, `None` will be
         returned instead.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate the current `value` when
         the method `validate` is called.
 
     Any other named parameter will be stored in `self.extra`.
     """
 
-    def __init__(self, items, clean=None, *validate, **kwargs):
+    def __init__(self, items, clean=None, validate=None, **kwargs):
+        validate = validate or []
         self.items = items
         self.clean = clean
-        super(_Select, self).__init__(*validate, **kwargs)
+        super(_Select, self).__init__(validate=validate, **kwargs)
 
     def get_items(self):
         return self.items() if callable(self.items) else self.items
@@ -620,17 +653,18 @@ class _SelectMulti(_Field):
         An optional function that takes a value and return a 'cleaned' version
         of it. If a value raise an exception it'll be filtered out from the
         final result.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate each of the selected values
         when the method `validate` is called.
 
     Any other named parameter will be stored in `self.extra`.
     """
 
-    def __init__(self, items, clean=None, *validate, **kwargs):
+    def __init__(self, items, clean=None, validate=None, **kwargs):
+        validate = validate or []
         self.items = items
         self.clean = clean
-        super(_SelectMulti, self).__init__(*validate, **kwargs)
+        super(_SelectMulti, self).__init__(validate=validate, **kwargs)
 
     def _get_value(self):
         return self._value if self._value and not self.hide_value else []
@@ -645,6 +679,11 @@ class _SelectMulti(_Field):
     def get_items(self):
         return self.items() if callable(self.items) else self.items
 
+    def __iter__(self):
+        items = self.get_items()
+        for item in items:
+            yield item
+
     def to_python(self):
         values = self._value
         if self.clean:
@@ -732,50 +771,55 @@ class _Collection(_Text):
         An optional function that takes a value and return a 'cleaned' version
         of it. If a value raise an exception it'll be filtered out from the
         final result.
-    :param *validate:
+    :param validate:
         An list of validators. This will evaluate each of the selected values
         when the method `validate` is called.
 
     Any other named parameter will be stored in `self.extra`.
     """
 
-    def __init__(self, sep=', ', filters=None, clean=None,
-            *validate, **kwargs):
+    def __init__(self, sep=', ', filters=None, clean=None, validate=None,
+            **kwargs):
+        validate = validate or []
         self.sep = sep
         filters = filters or []
         self.filters = [f() if inspect.isclass(f) else f for f in filters]
         self.clean = clean
-        super(_Collection, self).__init__(*validate, **kwargs)
+        super(_Collection, self).__init__(validate=validate, **kwargs)
 
     def _get_value(self):
-        return self.sep.join(self._value) \
-            if self._value and not self.hide_value else u''
+        if self.hide_value:
+            return u''
+        if self._value:
+            return self.sep.join(self._value)
+        return self.to_html()
 
     def _set_value(self, value):
-        if not isinstance(value, list):
-            value = [value]
-        self._value = value
+        self._value = list(value)
 
     value = property(_get_value, _set_value)
     
-    def to_html(self, python_value):
-        return python_value or []
+    def to_html(self):
+        value = self.python_value or []
+        return self.sep.join(value)
 
-    def to_python(self):
+    def clean_value(self, python_value):
         sep = r'/s*%s/s*' % self.sep.replace(' ', '')
-        values = self._value
+        values = python_value or []
         for f in self.filters:
             values = filter(f, values)
         if self.clean:
             values_ = []
             for value in values:
                 try:
-                    values_.append(self.clean(value))
+                    val = self.clean(value)
                 except Exception:
-                    pass
+                    continue
+                if val:
+                    values_.append(val)
             values = values_
-        return values
-
+        
+        return filter(lambda v: bool(v), values)
 
 
 #- Field factories
@@ -833,6 +877,6 @@ class Select(Field):
 class SelectMulti(Field):
     _class = _SelectMulti
 
-class Collection(Text):
+class Collection(Field):
     _class = _Collection
 
